# McStas configuration options
#
# In this file, you can set up configuration options
# used by the McStas perl tools.
#
# CFLAGS   - compiler flags (should not require changes)
# CFLAGS_PLACEMENT - if 1, compiler flags are added to the end of the compile cmd
# LDFLAGS  - linker flags (should not require changes)
# MCCODE   - code generator to run (mcstas/mcxtrace)
# CC       - c compiler command (should not requrire changes)
# TERMINAL - (Unix only) Command to launch mcplot with (xterm/konsole/...)
# PGPLOT   - location of PGPLOT
# TCLTK    - location of TCLTK
# PLOTTER  - Choice of default plotting backend, possible values:
#            - PGPLOT (Unix / Win32)
#            - Matlab (Unix / Win32)
#            - Matlab scriptfile (Unix / Win32)
# PGDEV      - Default device for PGPLOT (usually /xserv or /gw)
# GNUDEV     - Default device for GNUPLOT (x11/aqua/windows)
# EDITOR     - Choice of default built-in editor
#            0 - McStas CVS-080208 editor (simple, no extra perl modules req'd)
#            1 - CodeText editor with highlighting of McStas instrument code
#            2 - Default to external editor
# TKPALETTE  - Palette choice for mcgui & child dialogues
# TKFONT     - Font choice for mcgui & child dialogues
# EXTERNAL_EDITOR - default external editor, overriden by VISUAL or EDITOR variables
# BROWSER  - Default BROWSER, overriden by BROWSER variable
# VRMLVIEW - Default VRML BROWSER, overriden by VRMLVIEW variable
# PREFIX   - Prefix for commands (e.g. start or empty)
# SUFFIX   - Suffix for perl scripts (.pl)
# DATE     - date format
# AT	   - Command to detach simulation process (Unix only)
# MATLAB   - comand to launch matlab
# GNUPLOT  - command to launch gnuplot
# EXE      - executable suffix (out or exe)
# HOSTFILE - MPI/grid hosts/machine list
# MATLAB_COMMAND - options to execute matlab
# CODETEXT - check for Tk::CodeText availability
# PROCSIMPLE - check for Proc::Simple availability
# TEMP     - check for File::Temp availability
# PREC     - optimization precision (stop criterion)
# SSH      - SSH client
# GRID_PING - Ping hosts before gridding or not (1/0)
# GRID_FORCECOMPILE - Force compilation on grid nodes (0/1)
#           (For heterogenous clusters)
# CLUSTER - Choice of default clustering method
#          0 - No clustering
#          1 - Threads (multi-core)
#          2 - MPI (Clusters)
#          3 - SSH (grid)
# NODES   - Choice of defaults number of nodes (MPI/GRID/THREADS)
#
# MCGUI_CFLAGS - Choose whether CFLAGS should be applied in mcgui (0/1)
# MCGUI_PGMULTI - Choose if mcgui uses 3-pane view when running mcplot/PGPLOT (0/1)
#

use Config;

# HEADER END

%MCSTAS::mcstas_config = (
     CFLAGS => '-g -O2 -lm -O2 -pipe  -fstack-protector-strong -fno-strict-aliasing',
     CFLAGS_PLACEMENT => '1',
     LDFLAGS => '',
     MCCODE => 'mcstas',
     PKGNAME => 'McStas',
     PARTICLE=> 'neutron',
     CC => '/usr/bin/cc',
     AT => '/usr/bin/at',
     TERMINAL => '',
     EXTERNAL_EDITOR => '',
     BROWSER => '',
     VRMLVIEW => '',
     PGPLOT => '',
     PGDEV => '',
     GNUDEV => '',
     TCLTK => 'wish',
     SSH => 'ssh',
     SCP => 'scp',
     MPIRUN => '',
     MPICC => '',
     PREFIX => '',
     SUFFIX => '.pl',
     PLOTTER => '',
     RUNCMD => 'mc'.'run'.'.pl',
     PLOTCMD=> 'mc'.'plot'.'.pl',
     TRACECMD=> 'mc'.'display'.'.pl',
     DOCCMD=> 'mc'.'doc'.'.pl',
     EDITOR => 1,
     TKPALETTE => '#c6c6c0',
     TKFONT => '-*-arial-normal-r-*-*-*-120-*-*-*-*-*-*',
     MATLAB => 'matlab',
     GNUPLOT => '',
     MATLAB_COMMAND => '-nodesktop -nosplash',
     EXE => 'out',
     HOSTFILE => '',
     MODULES => '',
     CODETEXT => 'no',
     PROCSIMPLE => 'no',
     TEMP => 'no',
     NEXUS => '',
     HDFVIEW => 'hdfview',
     PREC => '1e-3',
     CLUSTER => 0,
     NODES => 2,
     MCGUI_CFLAGS => 0,
     MCGUI_PGMULTI => 0,
     GRID_PING => 0,
     GRID_FORCECOMPILE => 0,
     VERSION => '2.5'
    );

# FOOTER START

# **** installed site perl modules availability

my $perl_PGPLOT=0;
my $pdl=0;

if ($MCSTAS::mcstas_config{'PGPLOT'} ne "no") {
  $MCSTAS::mcstas_config{'PGPLOT'} = "no";  # unset it and then search pgperl
  foreach $inc (@INC) {
    my $where="$inc/PGPLOT.pm";
    if (-e $where) {
      $MCSTAS::mcstas_config{'PGPLOT'}=$inc; # we got pgplot+pgperl
      require PGPLOT;
      $perl_PGPLOT=$where;

    }
    $where="$inc/PDL.pm";
		if (-e "$where") {
			$pdl = $where; # we got PDL
		}
  }
  if (not $perl_PGPLOT) {
  	print STDERR "PGPLOT unactivated: install perl-PGPLOT first\n";
  }
  if (not $pdl) {
  	print STDERR "PGPLOT unactivated: install perl-PDL first\n";
  }
}


my $perl_tk=0;
if ($MCSTAS::mcstas_config{'TCLTK'} ne "no") {
  $MCSTAS::mcstas_config{'TCLTK'} = "no";  # unset it and then search perl-tk
  foreach $inc (@INC) {
    my $where="$inc/Tk.pm";
    if (-e $where) {
      $MCSTAS::mcstas_config{'TCLTK'} = $inc; # we got tk+perltk
      require Tk;
      $perl_tk = $where;
    }
  }
  if (not $perl_tk) {
  	print STDERR "McGUI not available: install perl-Tk first\n";
  }
}

# **** installed McStas perl modules availability

if (defined($MCSTAS::perl_modules) && -e $MCSTAS::perl_modules) {
  $MCSTAS::mcstas_config{'MODULES'} = "$MCSTAS::perl_modules";
}

my $textedit=0;
my $storable=0;

foreach $inc (@INC) {
  my $where="$inc/Tk/CodeText.pm";
  my $temp_avail="";
  if (-e "$where") {
    $MCSTAS::mcstas_config{'CODETEXT'} = $where; # we got Tk::CodeText
  }

  $where="$inc/Proc/Simple.pm";
  if (-e "$where") {
    $MCSTAS::mcstas_config{'PROCSIMPLE'} = $where; # we got Proc::Simple
  }

  $where="$inc/Temp.pm";
  if (-e $where) { $temp_avail=$where; }
  $where="$inc/File/Temp.pm";
  if (-e $where) { $temp_avail=$where; }
  if ($temp_avail ne "") {
    $MCSTAS::mcstas_config{'TEMP'} = $temp_avail;
  }

  $where="$inc/Tk/TextEdit.pm";
  if (-e "$where") {
    $textedit = $where; # we got CodeText+TextEdit
  }

  $where="$inc/Storable.pm";
  if (-e "$where") {
    $storable = $where; # we got CodeText+TextEdit
  }

  $where="$inc/Math/Amoeba.pm";
  if (-e "$where") {
    $MCSTAS::mcstas_config{'AMOEBA'} = $where; # we got Math::Amoeba
  }

}

if ($MCSTAS::mcstas_config{'CODETEXT'} ne "no") {
  if ($textedit eq 0 || $storable eq 0) {
    $MCSTAS::mcstas_config{'CODETEXT'} = "no";
  }
}

# check of editors availability
if ($MCSTAS::mcstas_config{'EDITOR'} == 1 && $MCSTAS::mcstas_config{'CODETEXT'} eq "no") {
  $MCSTAS::mcstas_config{'EDITOR'} = 0;
}

# **** update MCSTAS config from $ENV

if ($ENV{'BROWSER'}) {
  $MCSTAS::mcstas_config{'BROWSER'} = $ENV{'BROWSER'} ;
}

if ($ENV{'VRMLVIEW'}) {
  $MCSTAS::mcstas_config{'VRMLVIEW'} = $ENV{'VRMLVIEW'} ;
}

if ($ENV{'VISUAL'} || $ENV{'EDITOR'}) {
  $MCSTAS::mcstas_config{'EXTERNAL_EDITOR'} = $ENV{'VISUAL'} || $ENV{'EDITOR'};
}

if ($ENV{'MCSTAS_FORMAT'}) {
  $MCSTAS::mcstas_config{'PLOTTER'} = $ENV{'MCSTAS_FORMAT'};
}

# Check value of $MCSTAS::mcstas_config{'PLOTTER'} variable
if ($MCSTAS::mcstas_config{'PLOTTER'} =~ /PGPLOT|McStas/i) {
  if ($MCSTAS::mcstas_config{'PLOTTER'} =~ /binary|float|double/i) {
    $MCSTAS::mcstas_config{'PLOTTER'} =~ s/binary//; # PGPLOT binary format is invalid. solving.
    $MCSTAS::mcstas_config{'PLOTTER'} =~ s/float//;
    $MCSTAS::mcstas_config{'PLOTTER'} =~ s/double//;
  }
}

# Attempt to locate pgplot directory if unset.
$ENV{'PGPLOT_DIR'} = "/usr/local/pgplot" unless $ENV{'PGPLOT_DIR'};
$ENV{'PGPLOT_DEV'} = $MCSTAS::mcstas_config{'PGDEV'} unless $ENV{'PGPLOT_DEV'};
$MCSTAS::mcstas_config{'PGDEV'} = $ENV{'PGPLOT_DEV'};

# Override default CC/CFLAGS if MCSTAS/MCXTRACE_CC/CFLAGS defined
# - The first four sets are used on Windows, as the CMake/build-time variables are not correct for running
# - The second set is what mcgui (py) uses to communicate the settings
$MCSTAS::mcstas_config{'CC'} = $ENV{'MCSTAS_CC'} if defined($ENV{'MCSTAS_CC'});
$MCSTAS::mcstas_config{'CFLAGS'} = $ENV{'MCSTAS_CFLAGS'} if defined($ENV{'MCSTAS_CFLAGS'});
$MCSTAS::mcstas_config{'MPICC'} = $ENV{'MCSTAS_MPICC'} if defined($ENV{'MCSTAS_MPICC'});
$MCSTAS::mcstas_config{'MPIRUN'} = $ENV{'MCSTAS_MPIRUN'} if defined($ENV{'MCSTAS_MPIRUN'});
$MCSTAS::mcstas_config{'CC'} = $ENV{'MCSTAS_CC_OVERRIDE'} if defined($ENV{'MCSTAS_CC_OVERRIDE'});
$MCSTAS::mcstas_config{'CFLAGS'} = $ENV{'MCSTAS_CFLAGS_OVERRIDE'} if defined($ENV{'MCSTAS_CFLAGS_OVERRIDE'});
$MCSTAS::mcstas_config{'MPICC'} = $ENV{'MCSTAS_MPICC_OVERRIDE'} if defined($ENV{'MCSTAS_MPICC_OVERRIDE'});
$MCSTAS::mcstas_config{'MPIRUN'} = $ENV{'MCSTAS_MPIRUN_OVERRIDE'} if defined($ENV{'MCSTAS_MPIRUN_OVERRIDE'});

# **** update fields depending on system specific configuration

if ($Config{'osname'} ne 'MSWin32') { # Unix systems
  # Make sure that the current directory appears first in the path;
  # contrary to normal use, this is what the user expects here.
  $ENV{PATH} = $ENV{PATH} ? ".:$ENV{PATH}" : ".";
  my $HOME = $ENV{'HOME'} ? "$ENV{'HOME'}" : ".";
  my $hostfile = "";
  if ($ENV{'HOME'}) {
    if (-d "$HOME/.mcstas") {
      $hostfile = "$HOME/.mcstas/hosts";
    } else {
      $hostfile = "$HOME/.mcstas-hosts"
    }
  }
  else { $hostfile = ".mcstas-hosts"; }
  if (!-e $hostfile) {
    if ($ENV{'HOME'}) { $hostfile = "$HOME/mcstas-hosts"; }
    else { $hostfile = "mcstas-hosts"; }
    if (!-e $hostfile) {
      $hostfile = "$MCSTAS::sys_dir/tools/perl/mcstas-hosts";
      if (! -e $hostfile) { $hostfile = ""; }
    }
  }
  $MCSTAS::mcstas_config{'HOSTFILE'} = $hostfile;
} else {
  $MCSTAS::mcstas_config{'MCCODE'} = "mcstas";
}

1;  # return true for import/use/do

#
#   This file is part of the McStas neutron ray-trace simulation package
#   Copyright (C) 1997-2008, All rights reserved
#   Risoe National Laborartory, Roskilde, Denmark
#   Institut Laue Langevin, Grenoble, France
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; version 2 of the License.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
